@@ -693,7 +693,7 @@ rsvg_css_parse_font_family (const char * str, const char * inherit)
 		return str;
 }
 
-#if !defined(HAVE_STRTOK_R) && !GLIB_CHECK_VERSION(2, 3, 2)
+#if !defined(HAVE_STRTOK_R) /* && !GLIB_CHECK_VERSION(2, 3, 2) */
 
 static char *
 strtok_r(char *s, const char *delim, char **last)
@@ -734,9 +734,8 @@ strtok_r(char *s, const char *delim, char **last)
 gchar **
 rsvg_css_parse_list(const char * in_str, guint * out_list_len)
 {
-
-	/*the following code is defective because it creates blank entries when two splitting chars are next to each other*/
-#if 0//GLIB_CHECK_VERSION(2, 3, 2)
+	/* the following code is defective because it creates blank entries when two splitting chars are next to each other*/
+#if 0 /* GLIB_CHECK_VERSION(2, 3, 2) */
 
 	gchar ** string_array;
 	guint n;
@@ -754,7 +753,7 @@ rsvg_css_parse_list(const char * in_str, guint * out_list_len)
 #else
 
 	char *ptr, *tok;
-	char *str, *tmp;
+	char *str;
 
 	guint n = 0;
 	GSList * string_list = NULL;
@@ -763,18 +762,16 @@ rsvg_css_parse_list(const char * in_str, guint * out_list_len)
 	str = g_strdup (in_str);
     tok = strtok_r (str, ", \t", &ptr);
 	if (tok != NULL) {
-		tmp = g_strdup(tok);
-		if (strcmp(tmp, " "))
+		if (strcmp(tok, " ") != 0)
 			{
-				string_list = g_slist_prepend(string_list, tmp);
+				string_list = g_slist_prepend(string_list, g_strdup(tok));
 				n++;
 			}
 
 		while((tok = strtok_r (NULL, ", \t", &ptr)) != NULL) {
-			tmp = g_strdup(tok);
-			if (strcmp(tmp, " "))
+			if (strcmp(tok, " ") != 0)
 				{
-					string_list = g_slist_prepend(string_list, tmp);
+					string_list = g_slist_prepend(string_list, g_strdup(tok));
 					n++;
 				}
 		}
@@ -848,3 +845,45 @@ rsvg_css_parse_number_optional_number(const char * str,
   else
     *y = *x;
 }
+
+int rsvg_css_parse_aspect_ratio(const char * str)
+{
+	char ** elems;
+	guint nb_elems;
+
+	int ratio = RSVG_ASPECT_RATIO_NONE;
+
+	elems = rsvg_css_parse_list (str, &nb_elems);
+
+	if (elems && nb_elems)
+		{
+			guint i;
+
+			for (i = 0; i < nb_elems; i++) {
+				if (!strcmp (elems[i], "xMinYMin"))
+					ratio = RSVG_ASPECT_RATIO_XMIN_YMIN;
+				else if (!strcmp (elems[i], "xMidYMin"))
+					ratio = RSVG_ASPECT_RATIO_XMID_YMIN;
+				else if (!strcmp (elems[i], "xMaxYMin"))
+					ratio = RSVG_ASPECT_RATIO_XMAX_YMIN;
+				else if (!strcmp (elems[i], "xMinYMid"))
+					ratio = RSVG_ASPECT_RATIO_XMIN_YMID;
+				else if (!strcmp (elems[i], "xMidYMid"))
+					ratio = RSVG_ASPECT_RATIO_XMIN_YMID;
+				else if (!strcmp (elems[i], "xMaxYMid"))
+					ratio = RSVG_ASPECT_RATIO_XMAX_YMID;
+				else if (!strcmp (elems[i], "xMinYMax"))
+					ratio = RSVG_ASPECT_RATIO_XMIN_YMAX;
+				else if (!strcmp (elems[i], "xMidYMax"))
+					ratio = RSVG_ASPECT_RATIO_XMID_YMAX;
+				else if (!strcmp (elems[i], "xMaxYMax"))
+					ratio = RSVG_ASPECT_RATIO_XMAX_YMAX;
+				else if (!strcmp (elems[i], "slice"))
+					ratio |= RSVG_ASPECT_RATIO_SLICE;			
+			}
+
+			g_strfreev (elems);
+		}
+
+	return ratio;
+}
