@@ -31,15 +31,8 @@
 G_BEGIN_DECLS
 
 void rsvg_start_text (RsvgHandle *ctx, RsvgPropertyBag *atts);
-void rsvg_start_tspan (RsvgHandle *ctx, RsvgPropertyBag *atts);
 char * rsvg_make_valid_utf8 (const char *str);
 
-void 
-rsvg_text_render_text (RsvgHandle *ctx,
-					   RsvgState  *state,
-					   const char *text,
-					   const char *id);
-
 G_END_DECLS
 
 #endif /* RSVG_TEXT_H */
