@@ -21,6 +21,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <popt.h>
 
 #include <gtk/gtk.h>
@@ -42,16 +43,40 @@ struct _ViewerCbInfo
 	GdkPixbuf  * pixbuf;
 	GByteArray * svg_bytes;
 	GtkAccelGroup * accel_group;
+	char * base_uri;
 };
 
 static void
-rsvg_window_set_default_icon (GtkWindow * window, GdkPixbuf *icon)
+rsvg_window_set_default_icon (GtkWindow * window, GdkPixbuf *src)
 {
   GList *list;
-  
+  GdkPixbuf * icon;
+  gint width, height;
+
+  width = gdk_pixbuf_get_width(src);
+  height = gdk_pixbuf_get_height(src);
+
+  if(width > 128 || height > 128) {
+	  /* sending images greater than 128x128 has this nasty tendency to 
+		 cause broken pipe errors X11 Servers */
+	  if(width > height) {
+		  width = 0.5 + width * 128. / height;
+		  height = 128;
+	  } else {
+		  height = 0.5 + height * 128. / width;
+		  width = 128;
+	  }
+
+	  icon = gdk_pixbuf_scale_simple(src, width, height, GDK_INTERP_BILINEAR);
+  } else {
+	  icon = g_object_ref(G_OBJECT(src));
+  }
+
   list = g_list_prepend (NULL, icon);
   gtk_window_set_icon_list (window, list);
   g_list_free (list);
+
+  g_object_unref(G_OBJECT (icon));
 }
 
 #ifdef HAVE_GNOME_PRINT
@@ -115,7 +140,7 @@ print_pixbuf (GObject * ignored, gpointer user_data)
 					size_data.height = height;
 					size_data.keep_aspect_ratio = FALSE;
 
-					image = rsvg_pixbuf_from_data_with_size_data (info->svg_bytes->data, info->svg_bytes->len, &size_data, NULL);
+					image = rsvg_pixbuf_from_data_with_size_data (info->svg_bytes->data, info->svg_bytes->len, &size_data, info->base_uri, NULL);
 				} 
 			else 
 				g_object_ref (G_OBJECT (image));
@@ -365,12 +390,15 @@ quit_cb (GtkWidget *win, gpointer unused)
 static void
 view_pixbuf (ViewerCbInfo * info, int xid, const char * color)
 {
-	GtkWidget *win, *img;
-	gint width, height;
+	GtkWidget *win, *img, *scroll;
+	gint img_width, img_height, win_width, win_height;
 	GdkColor bg_color;
 
 	/* create toplevel window and set its title */
 
+	img_width = gdk_pixbuf_get_width (info->pixbuf);
+	img_height = gdk_pixbuf_get_height (info->pixbuf);
+
 	if(xid > 0)
 		{
 			GdkWindow *gdk_parent;
@@ -378,22 +406,22 @@ view_pixbuf (ViewerCbInfo * info, int xid, const char * color)
 			win = gtk_plug_new(0);
 
 			gdk_parent = gdk_window_foreign_new(xid);
-			gdk_window_get_geometry(gdk_parent, NULL, NULL, &width, &height, NULL);
+			gdk_window_get_geometry(gdk_parent, NULL, NULL, &win_width, &win_height, NULL);
 
-			/* so that button presses get registered */
+			/* so that button and key presses get registered */
 			gtk_widget_add_events (win, GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK);
 		}
 	else
 		{
 			win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-			width = MIN(gdk_pixbuf_get_width (info->pixbuf), DEFAULT_WIDTH) + 20;
-			height = MIN(gdk_pixbuf_get_height (info->pixbuf), DEFAULT_HEIGHT) + 20;
+
+			win_width = DEFAULT_WIDTH;
+			win_height = DEFAULT_HEIGHT;
 
 			gtk_window_set_title (GTK_WINDOW(win), _("SVG Viewer"));
-			rsvg_window_set_default_icon (GTK_WINDOW(win), info->pixbuf);
 		}
 
-	gtk_window_set_default_size(GTK_WINDOW(win), width, height);
+	rsvg_window_set_default_icon (GTK_WINDOW(win), info->pixbuf);
 
 	/* exit when 'X' is clicked */
 	g_signal_connect(G_OBJECT(win), "destroy", G_CALLBACK(quit_cb), NULL);
@@ -403,16 +431,21 @@ view_pixbuf (ViewerCbInfo * info, int xid, const char * color)
 	img = gtk_image_new_from_pixbuf (info->pixbuf);
 
 	/* pack the window with the image */
-	if(xid > 0)
-		gtk_container_add(GTK_CONTAINER(win), img);
-	else
-		{
-			GtkWidget *scroll;
+	if ((xid > 0 && (img_width > win_width || img_height > win_height))
+		|| (xid <= 0)) {
+		gtk_window_set_default_size(GTK_WINDOW(win), MIN(img_width, win_width),
+									MIN(img_height, win_height));
 
-			scroll = gtk_scrolled_window_new(NULL, NULL);
-			gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW(scroll), img);
-			gtk_container_add(GTK_CONTAINER(win), scroll);
-		}
+		scroll = gtk_scrolled_window_new(NULL, NULL);
+		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(scroll),
+										GTK_POLICY_AUTOMATIC,
+										GTK_POLICY_AUTOMATIC);
+		gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW(scroll), img);
+		gtk_container_add(GTK_CONTAINER(win), scroll);
+	} else {
+		gtk_container_add(GTK_CONTAINER(win), img);
+		gtk_window_set_default_size(GTK_WINDOW(win), img_width, img_height);
+	}
 
 	if (color && strcmp (color, "none") != 0)
 		{
@@ -459,11 +492,13 @@ main (int argc, char **argv)
 	poptContext popt_context;
 	double x_zoom = 1.0;
 	double y_zoom = 1.0;
-	double dpi = -1.0;
+	double dpi_x = -1.0;
+	double dpi_y = -1.0;
 	int width  = -1;
 	int height = -1;
 	int bVersion = 0;
 	char * bg_color = NULL;
+	char * base_uri = NULL;
 	int bKeepAspect = 0;
 
 	int xid = -1;
@@ -478,12 +513,14 @@ main (int argc, char **argv)
 			{ "xid",         'i',  POPT_ARG_INT,    &xid,         0, _("XWindow ID [for X11 embedding]"), _("<int>") },
 #endif
 			{ "stdin",       's',  POPT_ARG_NONE,   &from_stdin,  0, _("Read from stdin instead of a file"), NULL },
-			{ "dpi",         'd',  POPT_ARG_DOUBLE, &dpi,         0, _("Set the # of Pixels Per Inch"), _("<float>") },
+			{ "dpi-x",       'd',  POPT_ARG_DOUBLE, &dpi_x,       0, _("Set the # of Pixels Per Inch"), _("<float>") },
+			{ "dpi-y",       'p',  POPT_ARG_DOUBLE, &dpi_y,       0, _("Set the # of Pixels Per Inch"), _("<float>") },
 			{ "x-zoom",      'x',  POPT_ARG_DOUBLE, &x_zoom,      0, _("Set the x zoom factor"), _("<float>") },
 			{ "y-zoom",      'y',  POPT_ARG_DOUBLE, &y_zoom,      0, _("Set the y zoom factor"), _("<float>") },
 			{ "width",       'w',  POPT_ARG_INT,    &width,       0, _("Set the image's width"), _("<int>") },
 			{ "height",      'h',  POPT_ARG_INT,    &height,      0, _("Set the image's height"), _("<int>") },
 			{ "bg-color",    'b',  POPT_ARG_STRING, &bg_color,    0, _("Set the image background color (default: transparent)"), _("<string>") },
+			{ "base-uri",    'u',  POPT_ARG_STRING, &base_uri,    0, _("Set the base URI (default: none)"), _("<string>") },
 			{ "keep-aspect", 'k',  POPT_ARG_NONE,   &bKeepAspect, 0, _("Preserve the image's aspect ratio"), NULL },
 			{ "version",     'v',  POPT_ARG_NONE,   &bVersion,    0, _("Show version information"), NULL },
 			POPT_AUTOHELP
@@ -497,6 +534,7 @@ main (int argc, char **argv)
 	info.svg_bytes = NULL;
 	info.window = NULL;
 	info.popup_menu = NULL;
+	info.base_uri = base_uri;
 
 	popt_context = poptGetContext ("rsvg-view", argc, (const char **)argv, options_table, 0);
 	poptSetOtherOptionHelp (popt_context, _("[OPTIONS...] [file.svg]"));
@@ -526,8 +564,7 @@ main (int argc, char **argv)
 	/* initialize gtk+ */
 	gtk_init (&argc, &argv) ;
 
-	if (dpi > 0.)
-		rsvg_set_default_dpi (dpi);
+	rsvg_set_default_dpi_x_y (dpi_x, dpi_y);
 	
 	/* if both are unspecified, assume user wants to zoom the pixbuf in at least 1 dimension */
 	if (width == -1 && height == -1)
@@ -557,8 +594,10 @@ main (int argc, char **argv)
 	
 	if(from_stdin)
 		in = stdin;
-	else
+	else {
 		in = fopen (args[0], "rb");
+		base_uri = (char *)args[0];
+	}
 
 	if(!in)
 		{
@@ -598,7 +637,7 @@ main (int argc, char **argv)
 
 	poptFreeContext (popt_context);
 
-	info.pixbuf = rsvg_pixbuf_from_data_with_size_data (info.svg_bytes->data, info.svg_bytes->len, &size_data, &err);
+	info.pixbuf = rsvg_pixbuf_from_data_with_size_data (info.svg_bytes->data, info.svg_bytes->len, &size_data, base_uri, &err);
 
 	if (!info.pixbuf)
 		{
